package uicc.contactmanager;

/**
 * <p>
 *   A <code>ContactView</code> instance represents a contact in the contact manager.
 * </p>
 * <p>
 *   Each contact is a collection of fields.
 *   <br>
 *   Within a contact, each field is assigned a numeric identifier,
 *   that indicates the related field descriptor in the field descriptors list.
 * </p>
 * <p>
 *   A <code>ContactView</code> instance may successively represent multiple contacts
 *   throughout its lifetime.
 *   <br>
 *   It is good practice, for an application that needs to manipulate 
 *   contacts, to create one <code>ContactView</code> instance at installation, that will
 *   be re-used throughout the application's lifetime.
 * </p>
 * <p>
 *   A <code>ContactView</code> instance provides an enumeration scheme to enumerate all fields
 *   that belong to it, or a subset of them defined by specified filtering criteria.
 * </p>
 * <p>
 *   An instance is necessarily in one of the following states:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing contact in the contact list.
 *         This is the initial state after creation of a new instance.</li>
 *     <li>SELECTED: the instance represents a contact in the contacts list.</li>
 *   </ul>
 * </p>
 * <p>
 *   More detailed description of the behavior of a <code>ContactView</code> instance in these states 
 *   is given below:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing contact in the contact list.
 *         This is the initial state after creation of a new instance:
 *       <ul>
 *         <li>The identifier is -1.</li>
 *         <li>Fields may be modified.</li>
 *         <li>Field modifications will be retained until the instance is added to the
 *             contact list or an existing contact is selected.</li>
 *         <li>When the instance is added to the contacts list, it goes to state SELECTED.</li>
 *       </ul>
 *     </li>
 *     <li>SELECTED: the instance represents a contact in the contacts list.
 *       <ul>
 *         <li>The instance enters this state after successful execution of one of these methods:
 *           <ul>
 *             <li><code>ContactsListView.getContact()</code></li>
 *             <li><code>ContactsListView.nextContact()</code></li>
 *             <li><code>ContactsListView.addContact()</code></li>
 *           </ul>
 *         </li>
 *         <li>Any change to the contact's fields are applied to the contact.</li>
 *         <li>The instance goes back to the DESELECTED state at the following events:
 *           <ul>
 *             <li><code>release()</code> method invoked</li>
 *             <li>A RESET event occurred.</li>
 *           </ul>
 *         </li> 
 *       </ul>
 *     </li>
 *   </ul>
 * </p>
 * <p>
 *   The <code>ContactView</code> class also provides a factory method to create a <code>FieldView</code> instance.
 * </p>
 */
public interface ContactView {
    /**
     * <p>
     *   Returns the contact identifier.
     * </p>
     * <p>
     *   The identifier is -1 as long as the instance is in state DESELECTED.
     * </p>
     * 
     * @return contact identifier
     */
    public int getContactIdentifier();
    
    /**
     * <p>
     *   Factory method used to create a <code>FieldView</code> instance.
     * </p>
     * <p>
     *   The <code>FieldView</code> instance is owned by the calling application.
     * </p>
     * 
     * @return new <code>FieldView</code> instance
     */
    public FieldView createFieldInstance();

    /**
     * <p>
     *   Starts a new enumeration of all fields in the contact.
     * </p>
     * <ul>
     *   <li>A <code>ContactView</code> instance can process one enumeration at a time.</li>
     *   <li>The next element in the enumeration can be retrieved with the method 
     *       <code>selectNextField()</code>.</li>
     * </ul>
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code> with reason:
     *    <ul>
     *      <li><code>WRONG_STATE</code> if <code>ContactView</code> object is DESELECTED</li>
     *    </ul>
     */
    public void startEnumeration();

    /**
     * <p>
     *   Starts a new enumeration of fields in the contact that match specific criteria.
     * </p>
     * <p>
     *   Only fields that match a specific type, attributes and a value pattern will participate
     *   in the enumeration.
     *   <br>
     *   The <code>attributesMask</code> parameter is an integer, where each bit represents a specific attribute,
     *   as defined by the <code>FieldView.ATTR_*</code> constants. Bits set indicate the attributes that
     *   a field must have in order to participate in the enumeration: fields that don't have these attributes
     *   won't participate. Bits with value 0 are ignored.
     *   <br>
     *   The value pattern is a string that specifies the field values that should match.
     *   It may have three forms:
     *   <ul>
     *     <li>patterns that start with <code>'*'</code> indicate all strings ending with the 
     *         characters that follow the <code>'*'</code><br>
     *         example: pattern="*0603" indicates to search for fields that end with 0603</li>
     *     <li>patterns that end with <code>'*'</code> indicate all strings starting with the 
     *         characters that precede the <code>'*'</code><br>
     *         example: pattern="ste*", indicates to search for fields that start by "ste"</li>
     *     <li>patterns that neither start, nor end with <code>'*'</code>, specify the exact string 
     *         that the fields should match.<br>
     *         example: pattern="John", indicates to search for fields that match "John" exactly.</li>
     *   </ul>
     * </p>
     * <p>
     *   In addition to the filtering mechanism described here, all requirements of the method 
     *   <code>startEnumeration()</code> apply to this method.
     * </p>
     * 
     * @param type type of the selected fields
     * @param attributesMask mask indicating the required attributes of the selected fields
     * @param pattern Byte array containing the pattern that the selected fields should match. If null, no pattern-based filtering is done.
     * @param patternOffset Starting offset of the pattern in the pattern byte array.
     * @param patternLength Length of the pattern.
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if ContactView object is DESELECTED</li>
     *   </ul>
     */
    public void startEnumeration(short type, int attributesMask, byte [] pattern, short patternOffset, short patternLength);
    
    /**
     * <p>
     *   Checks if there are more fields in the current field enumeration.
     * </p>
     *  
     * @return true if there is at least one more field, false otherwise
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>ContactView</code> object is DESELECTED</li>
     *   </ul>
     */
    public boolean hasMoreFields();
    
    /**
     * <p>
     *   Selects the next field in the current enumeration, and set <code>fieldView</code> to represent
     *   the selected field.
     * </p>
     * <p>
     *   The order in which the fields are enumerated is unspecified.
     * </p>
     * 
     * @param fieldView <code>FieldView</code> object that must represent the next field in the enumeration.
     * 
     * @return field descriptor identifier, or -1 if the enumeration reached its end.
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_STATE</code> if <code>ContactView</code> object is DESELECTED</li>
     *   </ul>
     */
    public int selectNextField(FieldView fieldView);

    /**
     * <p>
     *   Selects the field from the contact whose descriptor 
     *   matches the specified <code>fieldDescriptorIdentifier</code>, 
     *   and set <code>fieldView</code> to represent the selected field.
     * </p>
     * 
     * @param fieldView <code>FieldView</code> object that must represent the matching field.
     * @param fieldDescriptorIdentifier field descriptor identifier of field to get.
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>, with reason:
     *   <ul>
     *     <li><code>WRONG_IDENTIFIER</code> if the field descriptors list contains no field with
     *         identifier <code>fieldDescriptorIdentifier</code>;</li>
     *     <li><code>WRONG_STATE</code> if <code>ContactView</code> object is DESELECTED.</li>
     *   </ul>
     */
    public void selectField(FieldView fieldView, int fieldDescriptorIdentifier);
    
    /**
     * <p>
     *   If a contact is currently SELECTED by the instance, deselect the contact.
     * </p>
     * <p>
     *   After a successful call to this method, the instance is in DESELECTED state.
     * </p>
     */
    public void deselect();
}
