package uicc.contactmanager;

/**
 * <p>
 *   A field descriptor describes a field that shall
 *   be present for all contacts in the contact manger.
 * </p>
 * <p>
 *   All contacts shall have a field that corresponds to each field descriptor
 *   listed in the field descriptors list.
 * </p>
 * <p>
 *   For each field descriptor, common properties are defined.
 *   These properties shall be fulfilled by the corresponding field for all
 *   contacts.<br>
 *   These properties are:
 *   <ul>
 *     <li>A field type (for example: an name, address, a number)</li>
 *     <li>A set of attributes.<br>
 *         Each attribute is either set or not set. All attributes are represented by an 
 *         integer value interpreted as a bit field.<br>
 *         The field descriptor lists the attributes that are mandatory for the corresponding
 *         field in all contacts. Each contact may set additional attributes, but shall not unset
 *         the attributes set here. The meanings of each attributes and their bit positions are
 *         defined in <code>FieldView.ATTR_*</code> constants.
 *     </li>
 *   </ul>
 * </p>
 * <p>
 *   The <code>FieldDescriptorView</code> class provides:
 *   <ul>
 *     <li>methods to set and retrieve the field type and attributes;</li>
 *     <li>methods to set and retrieve the field label;</li>
 *     <li>methods to set and retrieve the maximum size for the field values.</li>
 *   </ul>
 * </p>
 * <p>
 *   The <code>FieldDescriptorsListView</code> class and the <code>FieldDescriptorView</code> class should
 *   be used at application installation in order to define the contacts structure.
 * </p>
 * <p>
 *   A <code>FieldDescriptorView</code> instance may successively represent multiple field descriptors
 *   throughout its lifetime.
 *   <br>
 *   It is good practice, for an application that needs to manipulate 
 *   field descriptors, to create one <code>FieldDescriptorView</code> instance at installation, that will
 *   be re-used throughout the application's lifetime.
 * </p>
 * <p>
 *   An instance is necessarily in one of the following states:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing descriptor in the field descriptors list.
 *         This is the initial state after creation of a new instance.</li>
 *     <li>SELECTED: the instance represents a field descriptor in the descriptors list.</li>
 *   </ul>
 * </p>
 * <p>
 *   More detailed description of the behavior of a <code>FieldDescriptorView</code> instance in these states 
 *   is given below:
 *   <ul>
 *     <li>DESELECTED: the instance does not represent any existing descriptor in the field descriptors list.
 *         This is the initial state after creation of a new instance:
 *       <ul>
 *         <li>The identifier is -1.</li>
 *         <li>Attributes may be modified.</li>
 *         <li>Attributes modifications will be retained until the instance is added to the
 *             descriptors list or an existing descriptor is selected.</li>
 *         <li>When the instance is added to the descriptors list, it goes to state SELECTED.</li>
 *       </ul>
 *     </li>
 *     <li>SELECTED: the instance represents a field descriptor in the descriptors list.
 *       <ul>
 *         <li>The instance enters this state after successful execution of one of these methods:
 *           <ul>
 *             <li><code>FieldDescriptorList.getFieldDescriptor()</code></li>
 *             <li><code>FieldDescriptorList.nextFieldDescriptor()</code></li>
 *             <li><code>FieldDescriptorList.addFieldDescriptor()</code></li>
 *           </ul>
 *         </li>
 *         <li>Any change to the attributes shall be stored into the selected descriptor.</li>
 *         <li>The instance goes back to the DESELECTED state at the following events:
 *           <ul>
 *             <li><code>release()</code> method called.</li>
 *             <li>A RESET event occurred.</li>
 *           </ul>
 *         </li> 
 *       </ul>
 *     </li>
 *   </ul>
 * </p>
 * <p>
 *   The <code>FieldDescriptorView</code> instances are owned by the calling application.
 * </p>
 */
public interface FieldDescriptorView {

    /**
     * Sets the field type.
     * 
     * @param type field type
     */
    public void setType(short type);

    /**
     * Returns the field type. 
     * 
     * @return field type
     */
    public short getType();

    /**
     * <p>
     *   Sets the attributes that any related field must have.
     * </p>
     * <p>
     *   In any contact, the field related to this field descriptor must
     *   have at least the attributes defined by this method.
     * </p>
     * <p>
     *   It may have additional attributes not defined here.
     * </p>
     * <p>
     *   The <code>attributes</code> argument is an integer, where each bit
     *   represents an attribute. The attributes and their bit positions
     *   are defined in <code>FieldView.ATTR_*</code> constants.
     * </p>
     * 
     * @param attributes field attributes
     */
    public void setAttributes(int attributes);

    /**
     * Returns the attributes that any related field must have, as defined
     * by the setAttributes() method.
     * 
     * @return field attributes
     */
    public int getAttributes();

    /**
     * Sets the label of the field (text name of the field).
     * 
     * @param label field label
     * @param labelOffset offset in label buffer 
     * @param labelLength length of label
     * 
     */

    public void setLabel(byte[] label, short labelOffset,
            short labelLength);

    /**
     * Gets the label of the field (text name of the field).
     * 
     * @param label buffer to store label into
     * @param labelOffset offset in label buffer
     * @param maxLabelLength maximum length of the label 
     * 
     * @return length read
     */

    public short getLabel(byte[] label, short labelOffset,
            short maxLabelLength);

    /**
     * Sets the maximum size of the field values.
     * 
     * @param maximumSize the maximum field size
     */
    public void setFieldMaxSize(int maximumSize);

    /**
     * Returns the maximum field size.
     * 
     * @return maximum field size
     */
    public int getFieldMaxSize();

    /**
     * Returns the identifier of the field descriptor.
     * 
     * @return the field descriptor identifier, -1 if not included yet in a field descriptors list
     * 
     */
    public int getFieldDescriptorIdentifier();
    
    /**
     * <p>
     *   If the instance is SELECTED to a field descriptor, removes this bond.
     * </p>
     * <p>
     *   After a successful call to this method, the instance is in DESELECTED state.
     * </p>
     */
    public void deselect();
}